def convert_to_openapi_example(configuration_examples):
    return {
        k: {"summary": "{example}".format(example=k), "value": v}
        for k, v in configuration_examples.items()
    }


server_configuration_examples = {
    "emptyserver": {},
    "DellR740": {
        "model": {"type": "rack"},
        "configuration": {
            "cpu": {"units": 2, "name": "intel xeon gold 6134"},
            "ram": [{"units": 12, "capacity": 32}],
            "disk": [{"units": 1, "type": "ssd", "capacity": 400, "density": 50.6}],
        },
        "usage": {"avg_power": 300, "usage_location": "FRA"},
    },
}

server_configuration_examples_openapi = convert_to_openapi_example(
    server_configuration_examples
)

components_examples = {
    "cpu": {
        "intel xeon gold 6134": {
            "name": "intel xeon gold 6134",
        },
        "empty cpu": {},
        "custom cpu": {
            "core_units": 64,
            "tdp": 150,
            "die_size": 400,
            "usage": {"usage_location": "FRA"},
        },
    },
    "ssd": {
        "custom ssd": {
            "capacity": 500,
            "manufacturer": "Samsung",
        },
        "empty ssd": {},
    },
    "ram": {
        "custom ram": {"capacity": 32, "manufacturer": "Samsung", "process": 30.0},
        "empty ram": {},
    },
    "hdd": {"empty HDD": {}},
    "motherboard": {"empty motherboard": {}},
    "power_supply": {"custom power supply": {"unit_weight": 10}},
    "case": {"rack": {"case_type": "rack"}, "blade": {"case_type": "blade"}},
}

components_examples_openapi = {
    "cpu": convert_to_openapi_example(components_examples["cpu"]),
    "ram": convert_to_openapi_example(components_examples["ram"]),
    "ssd": convert_to_openapi_example(components_examples["ssd"]),
    "hdd": convert_to_openapi_example(components_examples["hdd"]),
    "motherboard": convert_to_openapi_example(components_examples["motherboard"]),
    "power_supply": convert_to_openapi_example(components_examples["power_supply"]),
    "case": convert_to_openapi_example(components_examples["case"]),
}

cloud_usage_example = {
    "usage_location": "FRA",
    "time_workload": [
        {"time_percentage": 50, "load_percentage": 0},
        {"time_percentage": 25, "load_percentage": 60},
        {"time_percentage": 25, "load_percentage": 100},
    ],
}

cloud_example = {
    "provider": "aws",
    "instance_type": "a1.4xlarge",
    "usage": {
        "usage_location": "FRA",
        "time_workload": [
            {"time_percentage": 50, "load_percentage": 0},
            {"time_percentage": 25, "load_percentage": 60},
            {"time_percentage": 25, "load_percentage": 100},
        ],
    },
}

cpu_consumption_profiles = {
    "cpu": {"name": "intel xeon gold 6134"},
    "workload": [
        {"load_percentage": 0, "power_watt": 50},
        {"load_percentage": 10, "power_watt": 100},
        {"load_percentage": 50, "power_watt": 180},
        {"load_percentage": 100, "power_watt": 245},
    ],
}

end_user_terminal = {
    "usage": {
        "use_time_ratio": 0.3,
        "usage_location": "FRA",
    }
}



from dataclasses import dataclass
from typing import Optional

import boaviztapi.utils.roundit as rd
from boaviztapi import config

WARNING_IMPORTANT_UNCERTAINTY = "Uncertainty from technical characteristics is very important. Results should be interpreted with caution (see min and max values)"
NOT_IMPLEMENTED = "not implemented"


@dataclass
class ImpactCriteria:
    name: str
    unit: str
    description: str
    method: Optional[str] = None


class Impact:
    def __init__(self, **kwargs):
        self.value = 0
        self.min = 0
        self.max = 0
        self.warnings = []

        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)

    def add_warning(self, warn):
        if warn not in self.warnings:
            self.warnings.append(warn)

    def to_json(self):
        json = {"value": self.rounded_value()}
        if self.min or self.min == 0:
            json["min"] = self.rounded_min()
        if self.max or self.max == 0:
            json["max"] = self.rounded_max()
        if self.warnings:
            json["warnings"] = sorted(self.warnings)

        return json

    def rounded_value(self):
        rd_value = rd.round_based_on_min_max(self.value, self.min, self.max)
        nb_sig_fig = rd.significant_number(rd_value)

        if nb_sig_fig > config["max_sig_fig"]:
            return rd.round_to_sigfig(rd_value, config["max_sig_fig"])
        elif rd_value == 0:
            self.add_warning(WARNING_IMPORTANT_UNCERTAINTY)
            return rd.round_to_sigfig(self.value, config["min_sig_fig"])
        else:
            return rd_value

    def rounded_min(self):
        return rd.round_to_sigfig(self.min, config["max_sig_fig"])

    def rounded_max(self):
        return rd.round_to_sigfig(self.max, config["max_sig_fig"])

    def allocate(self, duration, life_time):
        if duration > life_time.value:
            allocation_ratio, allocation_ratio_min, allocation_ratio_max = 1, 1, 1
        else:
            allocation_ratio = duration / life_time.value
            allocation_ratio_min = duration / life_time.max
            allocation_ratio_max = duration / life_time.min

        self.value = self.value * allocation_ratio
        self.min = self.min * allocation_ratio_min
        self.max = self.max * allocation_ratio_max


GWP = ImpactCriteria(name="gwp", unit="kgCO2eq", description="Total climate change")
ADP = ImpactCriteria(
    name="adp", unit="kgSbeq", description="Use of minerals and fossil ressources"
)
PE = ImpactCriteria(name="pe", unit="MJ", description="Consumption of primary energy")
GWPPb = ImpactCriteria(
    name="gwppb",
    unit="kg CO2 eq.",
    method="PEF",
    description="Climate change - Contribution of biogenic emissions",
)
GWPPf = ImpactCriteria(
    name="gwppf",
    unit="kg CO2 eq.",
    method="PEF",
    description="Climate change - Contribution of fossil fuel emissions",
)
GWPPlu = ImpactCriteria(
    name="gwpplu",
    unit="kg CO2 eq.",
    method="PEF",
    description="Climate change - Contribution of emissions from land use change",
)
IR = ImpactCriteria(
    name="ir",
    unit="kg U235 eq.",
    method="PEF",
    description="Emissions of radionizing substances",
)
LU = ImpactCriteria(
    name="lu", unit="No dimension", method="PEF", description="Land use"
)
ODP = ImpactCriteria(
    name="odp",
    unit="kg CFC-11 eq.",
    method="PEF",
    description="Depletion of the ozone layer",
)
PM = ImpactCriteria(
    name="pm",
    unit="Disease occurrence",
    method="PEF",
    description="Fine particle emissions",
)
POCP = ImpactCriteria(
    name="pocp",
    unit="kg NMVOC eq.",
    method="PEF",
    description="Photochemical ozone formation",
)
WU = ImpactCriteria(
    name="wu", unit="m3 eq.", method="PEF", description="Use of water resources"
)
MIPS = ImpactCriteria(
    name="mips", unit="kg", description="Material input per unit of services"
)
ADPe = ImpactCriteria(
    name="adpe",
    unit="kg SB eq.",
    method="PEF",
    description="Use of mineral and metal resources",
)
ADPf = ImpactCriteria(
    name="adpf",
    unit="MJ",
    method="PEF",
    description="Use of fossil resources (including nuclear)",
)
AP = ImpactCriteria(
    name="ap", unit="mol H+ eq.", method="PEF", description="Acidification"
)
CTUe = ImpactCriteria(
    name="ctue", unit="CTUe", method="PEF", description="Freshwater ecotoxicity"
)
CTUh_c = ImpactCriteria(
    name="ctuh_c",
    unit="CTUh",
    method="PEF",
    description="Human Toxicity - Carcinogenic Effects",
)
CTUh_nc = ImpactCriteria(
    name="ctuh_nc",
    unit="CTUh",
    method="PEF",
    description="Human toxicity - non-carcinogenic effects",
)
Epf = ImpactCriteria(
    name="epf",
    unit="kg P eq.",
    method="PEF",
    description="Eutrophication of freshwater",
)
Epm = ImpactCriteria(
    name="epm",
    unit="kg N eq.",
    method="PEF",
    description="Eutrophication of marine waters",
)
Ept = ImpactCriteria(
    name="ept", unit="mol N eq.", method="PEF", description="Terrestrial eutrophication"
)
FW = ImpactCriteria(
    name="fw", unit="m3", method="", description="Net use of freshwater"
)

IMPACT_CRITERIAS = {
    "gwp": GWP,
    "adp": ADP,
    "pe": PE,
    "gwppb": GWPPb,
    "gwppf": GWPPf,
    "gwpplu": GWPPlu,
    "ir": IR,
    "lu": LU,
    "odp": ODP,
    "pm": PM,
    "pocp": POCP,
    "wu": WU,
    "mips": MIPS,
    "adpe": ADPe,
    "adpf": ADPf,
    "ap": AP,
    "ctue": CTUe,
    "CTUh_c": CTUh_c,
    "CTUh_nc": CTUh_nc,
    "epf": Epf,
    "epm": Epm,
    "ept": Ept,
    "fw": FW,
}

EMBEDDED = "embedded"
USE = "use"

IMPACT_PHASES = [EMBEDDED, USE]


class ImpactFactor:
    def __init__(self, **kwargs):
        self.value = 0
        self.min = 0
        self.max = 0
        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)


class Assessable:
    def __init__(self, **kwargs):
        self._impacts = {}

    def get_impacts(self, selected_criteria):
        result = {}
        for criteria in selected_criteria:
            result[criteria] = {}
            result[criteria]["unit"] = IMPACT_CRITERIAS[criteria].unit
            result[criteria]["description"] = IMPACT_CRITERIAS[criteria].description
            for phase in IMPACT_PHASES:
                if (
                    criteria not in self._impacts
                    or phase not in self._impacts[criteria]
                    or self._impacts[criteria][phase] is None
                ):
                    result[criteria][phase] = NOT_IMPLEMENTED
                else:
                    result[criteria][phase] = self._impacts[criteria][phase].to_json()
        return result

    @property
    def impacts(self):
        return self._impacts

    @impacts.setter
    def impacts(self, impacts):
        self._impacts = impacts

    def add_impacts(self, impact, criteria, phase):
        if criteria not in self._impacts:
            self._impacts[criteria] = {}
            self._impacts[criteria]["unit"] = IMPACT_CRITERIAS[criteria].unit
            self._impacts[criteria]["description"] = IMPACT_CRITERIAS[
                criteria
            ].description
        if phase not in self._impacts[criteria]:
            self._impacts[criteria][phase] = {}
        self._impacts[criteria][phase] = impact



import os

import numpy as np
import pandas as pd

from boaviztapi import data_dir
from boaviztapi.model.impact import IMPACT_CRITERIAS

_electricity_emission_factors_df = pd.read_csv(
    os.path.join(data_dir, "crowdsourcing/electrical_mix.csv")
)
_iot_impact_factors_df = pd.read_csv(
    os.path.join(data_dir, "crowdsourcing/iot_factors_tide.csv"), header=1
)


def electical_mix_jsonify(electrical_mix):
    json = {"min-max": {}}
    for country_code in electrical_mix.columns:
        if (
            country_code == "english_name"
            or country_code == "french_name"
            or country_code == "unit"
            or country_code == "source"
            or country_code == "priority"
            or country_code == "reference_year"
            or country_code == "name"
        ):
            continue
        if country_code not in json:
            json[country_code] = {}
            json[country_code]["country"] = electrical_mix[country_code][0]
        for index, row in electrical_mix.iterrows():
            if row["name"] == "renewable_energy" or row["name"] == "country":
                continue
            if np.isnan(float(row[country_code])):
                continue
            if row["name"] in json[country_code]:
                if row["priority"] > json[country_code][row["name"]]["priority"]:
                    continue

            json[country_code][row["name"]] = {}
            json[country_code][row["name"]]["unit"] = row["unit"]
            json[country_code][row["name"]]["source"] = row["source"]
            json[country_code][row["name"]]["priority"] = row["priority"]
            json[country_code][row["name"]]["reference_year"] = row["reference_year"]
            json[country_code][row["name"]]["value"] = float(row[country_code])

            if row["name"] not in json["min-max"]:
                json["min-max"][row["name"]] = {}
                json["min-max"][row["name"]]["min"] = float(row[country_code])
                json["min-max"][row["name"]]["max"] = float(row[country_code])
            else:
                if float(row[country_code]) < json["min-max"][row["name"]]["min"]:
                    json["min-max"][row["name"]]["min"] = float(row[country_code])
                if float(row[country_code]) > json["min-max"][row["name"]]["max"]:
                    json["min-max"][row["name"]]["max"] = float(row[country_code])

    return {"electricity": json}


def iot_jsonify(df):
    json = {"IoT": {}}
    df[["functional_block", "hsl_level"]] = df["api_name"].str.split(" - ", expand=True)
    df.index.names = ["api_name"]
    for functional_block in df["functional_block"].unique():
        if (
            functional_block == "Name"
            or functional_block == "api_name"
            or functional_block == "Unit"
        ):
            continue
        json["IoT"][functional_block] = {}
        for hsl_level in df[df["functional_block"] == functional_block][
            "hsl_level"
        ].unique():
            if hsl_level is None:
                continue
            json["IoT"][functional_block][hsl_level] = {}
            json["IoT"][functional_block][hsl_level]["manufacture"] = {}
            json["IoT"][functional_block][hsl_level]["eol"] = {}
            json["IoT"][functional_block][hsl_level]["source"] = (
                "Adapted by Tide from 'Assessing the embodied carbon footprint of IoT edge devices with a bottom-up life-cycle approach', 2021; Thibault Pirson et David Bol (Université catholique de Louvain, ICTEAM/ECS, Louvain-la-Neuve, Belgique)"
            )

            for impact in IMPACT_CRITERIAS:
                if (
                    df[
                        (df["functional_block"] == functional_block)
                        & (df["hsl_level"] == hsl_level)
                    ].get(f"eol-{impact.name}")
                    is None
                ):
                    continue
                json["IoT"][functional_block][hsl_level]["manufacture"][impact.name] = (
                    float(
                        df[
                            (df["functional_block"] == functional_block)
                            & (df["hsl_level"] == hsl_level)
                        ]
                        .get(f"fab-{impact.name}")
                        .iloc[0]
                    )
                )
                json["IoT"][functional_block][hsl_level]["eol"][impact.name] = float(
                    df[
                        (df["functional_block"] == functional_block)
                        & (df["hsl_level"] == hsl_level)
                    ]
                    .get(f"eol-{impact.name}")
                    .iloc[0]
                )

    return json


if __name__ == "__main__":
    print(iot_jsonify(_iot_impact_factors_df))



from typing import List, Optional
from fastapi import APIRouter, Query, Body

from boaviztapi import config
from boaviztapi.dto.device.user_terminal import (
    Monitor,
    UsbStick,
    ExternalSSD,
    ExternalHDD,
    VrController,
)
from boaviztapi.routers.openapi_doc.descriptions import (
    all_archetype_user_terminals,
    all_peripheral_categories,
    get_archetype_config_desc,
    peripheral_description,
)
from boaviztapi.routers.openapi_doc.examples import end_user_terminal
from boaviztapi.routers.terminal_router import (
    user_terminal_impact,
    get_all_archetype_name,
    get_archetype_config,
)

peripheral_router = APIRouter(prefix="/v1/peripheral", tags=["peripheral"])


@peripheral_router.get("/all", description=all_peripheral_categories)
async def peripheral_get_all_categories():
    return {
        "monitor": "v1/peripheral/monitor",
        "usb_stick": "v1/peripheral/usb_stick",
        "external_ssd": "v1/peripheral/external_ssd",
        "external_hdd": "v1/peripheral/external_hdd",
        "vr_controller": "v1/peripheral/vr_controller",
    }


@peripheral_router.get("/monitor/archetypes", description=all_archetype_user_terminals)
async def monitor_get_all_archetype_name():
    return get_all_archetype_name("monitor")


@peripheral_router.get(
    "/monitor/archetype_config", description=get_archetype_config_desc
)
async def monitor_get_archetype_config(
    archetype: str = Query(example=config["default_monitor"]),
):
    return get_archetype_config(archetype)


@peripheral_router.post("/monitor", description=peripheral_description)
async def monitor_impact(
    monitor: Monitor = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_monitor"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=monitor,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get("/monitor", description=peripheral_description)
async def monitor_impact(
    archetype: str = config["default_monitor"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=Monitor(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get(
    "/usb_stick/archetypes", description=all_archetype_user_terminals
)
async def usb_stick_get_all_archetype_name():
    return get_all_archetype_name("usb_stick")


@peripheral_router.get(
    "/usb_stick/archetype_config", description=get_archetype_config_desc
)
async def usb_stick_get_archetype_config(
    archetype: str = Query(example=config["default_usb_stick"]),
):
    return get_archetype_config(archetype)


@peripheral_router.post("/usb_stick", description=peripheral_description)
async def usb_stick_impact(
    usb_stick: UsbStick = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_usb_stick"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=usb_stick,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get("/usb_stick", description=peripheral_description)
async def usb_stick_impact(
    archetype: str = config["default_usb_stick"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=UsbStick(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get(
    "/external_ssd/archetypes", description=all_archetype_user_terminals
)
async def external_ssd_get_all_archetype_name():
    return get_all_archetype_name("external_ssd")


@peripheral_router.get(
    "/external_ssd/archetype_config", description=get_archetype_config_desc
)
async def external_ssd_get_archetype_config(
    archetype: str = Query(example=config["default_external_ssd"]),
):
    return get_archetype_config(archetype)


@peripheral_router.post("/external_ssd", description=peripheral_description)
async def external_ssd_impact(
    external_ssd: ExternalSSD = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_external_ssd"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=external_ssd,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get("/external_ssd", description=peripheral_description)
async def external_ssd_impact(
    archetype: str = config["default_external_ssd"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=ExternalSSD(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get(
    "/external_hdd/archetypes", description=all_archetype_user_terminals
)
async def external_hdd_get_all_archetype_name():
    return get_all_archetype_name("external_hdd")


@peripheral_router.get(
    "/external_hdd/archetype_config", description=get_archetype_config_desc
)
async def external_hdd_get_archetype_config(
    archetype: str = Query(example=config["default_external_hdd"]),
):
    return get_archetype_config(archetype)


@peripheral_router.post("/external_hdd", description=peripheral_description)
async def external_hdd_impact(
    external_hdd: ExternalHDD = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_external_hdd"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=external_hdd,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get("/external_hdd", description=peripheral_description)
async def external_hdd_impact(
    archetype: str = config["default_external_hdd"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=ExternalHDD(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get(
    "/vr_controller/archetypes", description=all_archetype_user_terminals
)
async def vr_controller_get_all_archetype_name():
    return get_all_archetype_name("vr_controller")


@peripheral_router.get(
    "/vr_controller/archetype_config", description=get_archetype_config_desc
)
async def vr_controller_get_archetype_config(
    archetype: str = Query(example=config["default_vr_controller"]),
):
    return get_archetype_config(archetype)


@peripheral_router.post("/vr_controller", description=peripheral_description)
async def vr_controller_impact_from_configuration(
    vr_controller: VrController = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_vr_controller"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=vr_controller,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@peripheral_router.get("/vr_controller", description=peripheral_description)
async def vr_controller_impact(
    archetype: str = config["default_vr_controller"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=VrController(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )



import os

import pandas as pd
import toml
from fastapi import APIRouter, Query

from boaviztapi.dto.component.cpu import CPU
from boaviztapi.model import impact
from boaviztapi.model.component import ComponentCase
from boaviztapi.model.component.cpu import attributes_from_cpu_name
from boaviztapi.routers.openapi_doc.descriptions import (
    country_code,
    cpu_family,
    cpu_model_range,
    ssd_manufacturer,
    ram_manufacturer,
    case_type,
    name_to_cpu,
    cpu_names,
    impacts_criteria,
)
from boaviztapi.service.factor_provider import get_available_countries
from boaviztapi.utils.get_version import get_version_from_pyproject

utils_router = APIRouter(prefix="/v1/utils", tags=["utils"])

data_dir = os.path.join(os.path.dirname(__file__), "../data")
_cpu_specs = pd.read_csv(os.path.join(data_dir, "crowdsourcing/cpu_specs.csv"))
_ssd_manuf = pd.read_csv(os.path.join(data_dir, "crowdsourcing/ssd_manufacture.csv"))
_ram_manuf = pd.read_csv(os.path.join(data_dir, "crowdsourcing/ram_manufacture.csv"))


@utils_router.get("/version", description="Get the version of the API")
async def version():
    return get_version_from_pyproject()


@utils_router.get("/country_code", description=country_code)
async def utils_get_all_countries():
    return get_available_countries()


@utils_router.get("/cpu_family", description=cpu_family)
async def utils_get_all_cpu_family():
    df = _cpu_specs[_cpu_specs["code_name"].notna()]
    return [*df["code_name"].unique()]


@utils_router.get("/cpu_model_range", description=cpu_model_range)
async def utils_get_all_cpu_model_range():
    df = _cpu_specs[_cpu_specs["model_range"].notna()]
    return [*df["model_range"].unique()]


@utils_router.get("/ssd_manufacturer", description=ssd_manufacturer)
async def utils_get_all_ssd_manufacturer():
    df = _ssd_manuf[_ssd_manuf["manufacturer"].notna()]

    return [*df["manufacturer"].unique()]


@utils_router.get("/ram_manufacturer", description=ram_manufacturer)
async def utils_get_all_ram_manufacturer():
    df = _ram_manuf[_ram_manuf["manufacturer"].notna()]

    return [*df["manufacturer"].unique()]


@utils_router.get("/case_type", description=case_type)
async def utils_get_all_case_type():
    return ComponentCase.AVAILABLE_CASE_TYPE


@utils_router.get("/name_to_cpu", description=name_to_cpu)
async def name_to_cpu(cpu_name: str = Query(example="Intel Core i7-9700K")):
    cpu_attributes = attributes_from_cpu_name(cpu_name)
    if cpu_attributes is not None:
        (
            name,
            manufacturer,
            code_name,
            model_range,
            tdp,
            cores,
            threads,
            die_size,
            die_size_source,
            source,
        ) = cpu_attributes
        return CPU(
            family=code_name,
            name=name,
            tdp=tdp,
            core_units=cores,
            die_size=die_size,
            model_range=model_range,
            manufacturer=manufacturer,
        )
    else:
        return f"CPU name {cpu_name} is not found in our database"


@utils_router.get("/cpu_name", description=cpu_names)
async def utils_get_all_cpu_name():
    df = _cpu_specs[_cpu_specs["name"].notna()]
    return [*df["name"].unique()]


@utils_router.get("/impact_criteria", description=impacts_criteria)
async def utils_get_all_impacts_criteria():
    return impact.IMPACT_CRITERIAS



import os
from typing import List, Optional

from fastapi import APIRouter, Query, Body, HTTPException

from boaviztapi import config, data_dir
from boaviztapi.dto.device.user_terminal import (
    UserTerminal,
    VrHeadset,
    mapper_user_terminal,
    Laptop,
    Desktop,
    Smartphone,
    Television,
    Tablet,
    Box,
)
from boaviztapi.routers.openapi_doc.descriptions import (
    all_archetype_user_terminals,
    all_terminal_categories,
    get_archetype_config_desc,
    terminal_description,
)
from boaviztapi.routers.openapi_doc.examples import end_user_terminal
from boaviztapi.service.archetype import (
    get_user_terminal_archetype,
    get_device_archetype_lst_with_type,
)
from boaviztapi.service.impacts_computation import compute_impacts
from boaviztapi.service.verbose import verbose_device

terminal_router = APIRouter(prefix="/v1/terminal", tags=["terminal"])


@terminal_router.get("/all", description=all_terminal_categories)
async def terminal_get_all_categories():
    return {
        "laptop": "v1/terminal/laptop",
        "desktop": "v1/terminal/desktop",
        "smartphone": "v1/terminal/smartphone",
        "television": "v1/terminal/television",
        "tablet": "v1/terminal/tablet",
        "box": "v1/terminal/box",
        "vr_headset": "v1/terminal/vr_headset",
    }


@terminal_router.get("/laptop/archetypes", description=all_archetype_user_terminals)
async def laptop_get_all_archetype_name():
    return get_all_archetype_name("laptop")


@terminal_router.get("/laptop/archetype_config", description=get_archetype_config_desc)
async def laptop_get_archetype_config(
    archetype: str = Query(example=config["default_laptop"]),
):
    return get_archetype_config(archetype)


@terminal_router.post("/laptop", description=terminal_description)
async def laptop_impact(
    laptop: Laptop = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_laptop"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=laptop,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/laptop", description=terminal_description)
async def laptop_impact(
    archetype: str = config["default_laptop"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=Laptop(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/desktop/archetypes", description=all_archetype_user_terminals)
async def desktop_get_all_archetype_name():
    return get_all_archetype_name("desktop")


@terminal_router.get("/desktop/archetype_config", description=get_archetype_config_desc)
async def desktop_get_archetype_config(
    archetype: str = Query(example=config["default_desktop"]),
):
    return get_archetype_config(archetype)


@terminal_router.post("/desktop", description=terminal_description)
async def desktop_impact(
    desktop: Desktop = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_desktop"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=desktop,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/desktop", description=terminal_description)
async def desktop_impact(
    archetype: str = config["default_desktop"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=Desktop(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/smartphone/archetypes", description=all_archetype_user_terminals)
async def smartphone_get_all_archetype_name():
    return get_all_archetype_name("smartphone")


@terminal_router.get(
    "/smartphone/archetype_config", description=get_archetype_config_desc
)
async def smartphone_get_archetype_config(
    archetype: str = Query(example=config["default_smartphone"]),
):
    return get_archetype_config(archetype)


@terminal_router.post("/smartphone", description=terminal_description)
async def smartphone_impact(
    smartphone: Smartphone = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_smartphone"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=smartphone,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/smartphone", description=terminal_description)
async def smartphone_impact(
    archetype: str = config["default_smartphone"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=Smartphone(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/tablet/archetypes", description=all_archetype_user_terminals)
async def tablet_get_all_archetype_name():
    return get_all_archetype_name("tablet")


@terminal_router.get("/tablet/archetype_config", description=get_archetype_config_desc)
async def tablet_get_archetype_config(
    archetype: str = Query(example=config["default_tablet"]),
):
    return get_archetype_config(archetype)


@terminal_router.post("/tablet", description=terminal_description)
async def tablet_impact(
    tablet: Tablet = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_tablet"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=tablet,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/tablet", description=terminal_description)
async def tablet_impact(
    archetype: str = config["default_tablet"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=Tablet(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/television/archetypes", description=all_archetype_user_terminals)
async def television_get_all_archetype_name():
    return get_all_archetype_name("television")


@terminal_router.get(
    "/television/archetype_config", description=get_archetype_config_desc
)
async def television_get_archetype_config(
    archetype: str = Query(example=config["default_television"]),
):
    return get_archetype_config(archetype)


@terminal_router.post("/television", description=terminal_description)
async def television_impact(
    television: Television = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_television"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=television,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/television", description=terminal_description)
async def television_impact(
    archetype: str = config["default_television"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=Television(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/box/archetypes", description=all_archetype_user_terminals)
async def box_get_all_archetype_name():
    return get_all_archetype_name("box")


@terminal_router.get("/box/archetype_config", description=get_archetype_config_desc)
async def box_get_archetype_config(
    archetype: str = Query(example=config["default_box"]),
):
    return get_archetype_config(archetype)


@terminal_router.post("/box", description=terminal_description)
async def box_impact(
    box: Box = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_box"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=box,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/box", description=terminal_description)
async def box_impact(
    archetype: str = config["default_box"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=Box(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/vr_headset/archetypes", description=all_archetype_user_terminals)
async def vr_headset_get_all_archetype_name():
    return get_all_archetype_name("vr_headset")


@terminal_router.get(
    "/vr_headset/archetype_config", description=get_archetype_config_desc
)
async def vr_headset_get_archetype_config(
    archetype: str = Query(example=config["default_vr_headset"]),
):
    return get_archetype_config(archetype)


@terminal_router.post("/vr_headset", description=terminal_description)
async def vr_headset_impact_from_configuration(
    vr_headset: VrHeadset = Body(None, example=end_user_terminal),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    archetype: str = config["default_vr_headset"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=vr_headset,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


@terminal_router.get("/vr_headset", description=terminal_description)
async def vr_headset_impact(
    archetype: str = config["default_vr_headset"],
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    return await user_terminal_impact(
        user_terminal_dto=VrHeadset(),
        verbose=verbose,
        duration=duration,
        criteria=criteria,
        archetype=archetype,
    )


async def user_terminal_impact(
    user_terminal_dto: UserTerminal,
    archetype: str,
    verbose: bool,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
) -> dict:
    archetype_config = get_user_terminal_archetype(archetype)

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    device = mapper_user_terminal(user_terminal_dto, archetype=archetype_config)

    if duration is None:
        duration = device.usage.hours_life_time.value

    impacts = compute_impacts(
        model=device, selected_criteria=criteria, duration=duration
    )

    if verbose:
        return {
            "impacts": impacts,
            "verbose": verbose_device(
                device, selected_criteria=criteria, duration=duration
            ),
        }

    return {"impacts": impacts}


def get_all_archetype_name(name: str):
    result = get_device_archetype_lst_with_type(
        os.path.join(data_dir, "archetypes/user_terminal.csv"), name.lower()
    )
    if not result:
        return None
    return result


def get_archetype_config(archetype: str):
    result = get_user_terminal_archetype(archetype)
    if not result:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")
    return result



import os
from typing import List, Optional

import pandas as pd

from fastapi import APIRouter, Query, Body, HTTPException

from boaviztapi import config, data_dir
from boaviztapi.dto.device import Cloud
from boaviztapi.dto.device.device import mapper_cloud_instance
from boaviztapi.model.services.cloud_instance import ServiceCloudInstance
from boaviztapi.routers.openapi_doc.descriptions import (
    cloud_provider_description,
    all_default_cloud_instances,
    all_default_cloud_providers,
    get_instance_config,
)
from boaviztapi.routers.openapi_doc.examples import cloud_example
from boaviztapi.service.archetype import (
    get_cloud_instance_archetype,
    get_device_archetype_lst,
)
from boaviztapi.service.impacts_computation import compute_impacts
from boaviztapi.service.verbose import verbose_cloud

cloud_router = APIRouter(prefix="/v1/cloud", tags=["cloud"])


@cloud_router.get("/instance/instance_config", description=get_instance_config)
async def get_archetype_config(
    provider: str = Query(
        config["default_cloud_provider"], example=config["default_cloud_provider"]
    ),
    instance_type: str = Query(
        config["default_cloud_instance"], example=config["default_cloud_instance"]
    ),
):
    result = get_cloud_instance_archetype(instance_type, provider)
    if not result:
        raise HTTPException(
            status_code=404, detail=f"{instance_type} at {provider} not found"
        )
    return result


@cloud_router.post("/instance", description=cloud_provider_description)
async def instance_cloud_impact(
    cloud_instance: Cloud = Body(None, example=cloud_example),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    instance_archetype = get_cloud_instance_archetype(
        cloud_instance.instance_type, cloud_instance.provider
    )

    if not instance_archetype:
        raise HTTPException(
            status_code=404,
            detail=f"{cloud_instance.instance_type} at {cloud_instance.provider} not found",
        )

    instance_model = mapper_cloud_instance(cloud_instance, archetype=instance_archetype)

    return await cloud_instance_impact(
        cloud_instance=instance_model,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
    )


@cloud_router.get("/instance", description=cloud_provider_description)
async def instance_cloud_impact(
    provider: str = Query(
        config["default_cloud_provider"], example=config["default_cloud_provider"]
    ),
    instance_type: str = Query(
        config["default_cloud_instance"], example=config["default_cloud_instance"]
    ),
    verbose: bool = True,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
):
    cloud_instance = Cloud()
    cloud_instance.usage = {}
    instance_archetype = get_cloud_instance_archetype(instance_type, provider)

    if not instance_archetype:
        raise HTTPException(
            status_code=404, detail=f"{instance_type} at {provider} not found"
        )

    instance_model = mapper_cloud_instance(cloud_instance, archetype=instance_archetype)

    return await cloud_instance_impact(
        cloud_instance=instance_model,
        verbose=verbose,
        duration=duration,
        criteria=criteria,
    )


@cloud_router.get("/instance/all_instances", description=all_default_cloud_instances)
async def server_get_all_archetype_name(provider: str = Query(None, example="aws")):
    if not os.path.exists(data_dir + "/archetypes/cloud/" + provider + ".csv"):
        raise HTTPException(
            status_code=404,
            detail=f"No available data for this cloud provider ({provider})",
        )
    return get_device_archetype_lst(
        os.path.join(data_dir, "archetypes/cloud/" + provider + ".csv")
    )


@cloud_router.get("/instance/all_providers", description=all_default_cloud_providers)
async def server_get_all_provider_name():
    df = pd.read_csv(os.path.join(data_dir, "archetypes/cloud/providers.csv"))
    return df["provider.name"].tolist()


async def cloud_instance_impact(
    cloud_instance: ServiceCloudInstance,
    verbose: bool,
    duration: Optional[float] = config["default_duration"],
    criteria: List[str] = Query(config["default_criteria"]),
) -> dict:
    if duration is None:
        duration = cloud_instance.platform.usage.hours_life_time.value

    impacts = compute_impacts(
        model=cloud_instance, selected_criteria=criteria, duration=duration
    )

    if verbose:
        return {
            "impacts": impacts,
            "verbose": verbose_cloud(
                cloud_instance, selected_criteria=criteria, duration=duration
            ),
        }
    return {"impacts": impacts}



from typing import Tuple, Union, Optional

from boaviztapi import config
from boaviztapi.model import ComputedImpacts
from boaviztapi.model.device.server import DeviceServer
from boaviztapi.model.services.cloud_instance import Service, ServiceCloudInstance
from boaviztapi.model.component import (
    ComponentCPU,
    ComponentCase,
    ComponentPowerSupply,
    ComponentRAM,
    Component,
    ComponentAssembly,
    ComponentHDD,
    ComponentSSD,
    ComponentMotherboard,
)
from boaviztapi.model.component.functional_block import ComponentFunctionalBlock
from boaviztapi.model.device import Device
from boaviztapi.model.device.iot import DeviceIoT
from boaviztapi.model.impact import (
    ImpactFactor,
    IMPACT_PHASES,
    IMPACT_CRITERIAS,
    Impact,
    USE,
)
from boaviztapi.service.factor_provider import get_impact_factor, get_iot_impact_factor


def compute_single_impact(
    model: Union[Component, Device, Service],
    phase: str,
    criteria: str,
    duration: Union[int, str] = config["default_duration"],
    allocation: float = 1,
) -> Optional[Impact]:
    try:
        impact_function = get_impact_function(model, phase)

        impact, min_impact, max_impact, warnings = impact_function(
            criteria, duration, model
        )

        result = Impact(
            value=impact * allocation,
            min=min_impact * allocation,
            max=max_impact * allocation,
            warnings=list(set(warnings)),
        )

        model.add_impacts(result, criteria, phase)

        return result
    except (AttributeError, NotImplementedError):
        model.add_impacts(None, criteria, phase)
        return None


def compute_impacts(
    model: Union[Component, Device, Service],
    selected_criteria=config["default_criteria"],
    duration=config["default_duration"],
) -> dict:
    for c in IMPACT_CRITERIAS.keys():
        criteria = IMPACT_CRITERIAS[c]
        if "all" not in selected_criteria:
            if criteria.name not in selected_criteria:
                continue
        for phase in IMPACT_PHASES:
            compute_single_impact(model, phase, criteria.name, duration)

    return model.get_impacts(selected_criteria)


def get_impact_function(model: Union[Component, Device, Service], phase: str):
    return impacts_functions[model.NAME][phase]


def not_implemented_function(
    impact_type: str, duration: int, model: Union[Component, Device, Service]
):
    raise NotImplementedError


def simple_impact_use(
    impact_type: str, duration: int, model: Union[Component, Device, Service]
) -> ComputedImpacts:
    if not model.usage.avg_power.has_value():
        raise NotImplementedError

    impact_factor = model.usage.elec_factors[impact_type]

    impacts = (
        impact_factor.value
        * (model.usage.avg_power.value / 1000)
        * model.usage.use_time_ratio.value
        * duration
        * model.units.value
    )
    max_impact = (
        impact_factor.max
        * (model.usage.avg_power.max / 1000)
        * model.usage.use_time_ratio.min
        * duration
        * model.units.max
    )
    min_impact = (
        impact_factor.min
        * (model.usage.avg_power.min / 1000)
        * model.usage.use_time_ratio.max
        * duration
        * model.units.min
    )

    return impacts, min_impact, max_impact, []


def simple_embedded(
    impact_type: str, duration: int, model: [Device, Component, Service]
) -> ComputedImpacts:
    if hasattr(model, "type") and model.type is not None:
        impact = Impact(
            value=float(
                get_impact_factor(item=model.NAME, impact_type=impact_type)[
                    model.type.value
                ]["impact"]
            )
            * model.units.value,
            min=float(
                get_impact_factor(item=model.NAME, impact_type=impact_type)[
                    model.type.value
                ]["impact"]
            )
            * model.units.min,
            max=float(
                get_impact_factor(item=model.NAME, impact_type=impact_type)[
                    model.type.value
                ]["impact"]
            )
            * model.units.max,
        )
    else:
        impact = Impact(
            value=float(
                get_impact_factor(item=model.NAME, impact_type=impact_type)["impact"]
            )
            * model.units.value,
            min=float(
                get_impact_factor(item=model.NAME, impact_type=impact_type)["impact"]
            )
            * model.units.min,
            max=float(
                get_impact_factor(item=model.NAME, impact_type=impact_type)["impact"]
            )
            * model.units.max,
        )

    impact.allocate(duration, model.usage.hours_life_time)

    warnings = ["Generic data used for impact calculation."]

    return impact.value, impact.min, impact.max, warnings


def cpu_impact_use(
    impact_type: str, duration: int, cpu: ComponentCPU
) -> ComputedImpacts:
    impact_factor = cpu.usage.elec_factors[impact_type]

    if not cpu.usage.avg_power.is_set():
        modeled_consumption = cpu.model_power_consumption()
        cpu.usage.avg_power.set_completed(
            modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max,
        )

    impact = Impact(
        value=impact_factor.value
        * (cpu.usage.avg_power.value / 1000)
        * cpu.usage.use_time_ratio.value
        * duration,
        min=impact_factor.min
        * (cpu.usage.avg_power.min / 1000)
        * cpu.usage.use_time_ratio.min
        * duration,
        max=impact_factor.max
        * (cpu.usage.avg_power.max / 1000)
        * cpu.usage.use_time_ratio.max
        * duration,
    )

    return impact.value, impact.min, impact.max, []


def cpu_impact_embedded(
    impact_type: str, duration: int, cpu: ComponentCPU
) -> ComputedImpacts:
    cpu_die_impact = Impact(
        value=get_impact_factor(item="cpu", impact_type=impact_type)["die_impact"],
        min=get_impact_factor(item="cpu", impact_type=impact_type)["die_impact"],
        max=get_impact_factor(item="cpu", impact_type=impact_type)["die_impact"],
    )
    cpu_impact = Impact(
        value=get_impact_factor(item="cpu", impact_type=impact_type)["impact"],
        min=get_impact_factor(item="cpu", impact_type=impact_type)["impact"],
        max=get_impact_factor(item="cpu", impact_type=impact_type)["impact"],
    )

    impact = Impact(
        value=(cpu.die_size.value * cpu_die_impact.value + cpu_impact.value)
        * cpu.units.value,
        min=(cpu.die_size.min * cpu_die_impact.min + cpu_impact.min) * cpu.units.min,
        max=(cpu.die_size.max * cpu_die_impact.max + cpu_impact.max) * cpu.units.max,
    )

    impact.allocate(duration, cpu.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def assembly_impact_embedded(
    impact_type: str, duration: int, model: ComponentAssembly
) -> ComputedImpacts:
    impact = Impact(
        value=get_impact_factor(item="assembly", impact_type=impact_type)["impact"]
        * model.units.value,
        min=get_impact_factor(item="assembly", impact_type=impact_type)["impact"]
        * model.units.min,
        max=get_impact_factor(item="assembly", impact_type=impact_type)["impact"]
        * model.units.max,
    )

    impact.allocate(duration, model.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def casing_impact_embedded(
    impact_type: str, duration: int, case: ComponentCase
) -> ComputedImpacts:
    if case.case_type.value == "rack":
        computed_impact = impact_manufacture_rack(impact_type, case)
    elif case.case_type.value == "blade":
        computed_impact = impact_manufacture_blade(impact_type, case)
    else:
        computed_impact = impact_manufacture_rack(impact_type, case)

    impact = Impact(
        value=computed_impact[0] * case.units.value,
        min=computed_impact[1] * case.units.min,
        max=computed_impact[2] * case.units.max,
    )

    impact.allocate(duration, case.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def impact_manufacture_rack(impact_type: str, case: ComponentCase) -> ComputedImpacts:
    impact_factor = Impact(
        value=get_impact_factor(item="case", impact_type=impact_type)["rack"]["impact"],
        min=get_impact_factor(item="case", impact_type=impact_type)["rack"]["impact"],
        max=get_impact_factor(item="case", impact_type=impact_type)["rack"]["impact"],
    )

    if case.case_type.is_archetype() and case.case_type.value == "rack":
        blade_impact = impact_manufacture_blade(impact_type, case)
        if blade_impact[0] > impact_factor.value:
            return (
                impact_factor.value,
                impact_factor.min,
                blade_impact[2],
                ["End of life is not included in the calculation"],
            )
        else:
            return (
                impact_factor.value,
                blade_impact[1],
                impact_factor.max,
                ["End of life is not included in the calculation"],
            )
    return (
        impact_factor.value * case.units.value,
        impact_factor.min * case.units.min,
        impact_factor.max * case.units.max,
        ["End of life is not included in the calculation"],
    )


def impact_manufacture_blade(impact_type: str, case: ComponentCase) -> ComputedImpacts:
    impact_blade_server, impact_blade_16_slots = get_impact_constants_blade(impact_type)

    impact = compute_impact_manufacture_blade(
        impact_blade_server, impact_blade_16_slots
    )

    if case.case_type.is_archetype() and case.case_type.value == "blade":
        rack_impact = impact_manufacture_rack(impact_type, case)
        if rack_impact[0] > impact.value:
            return (
                impact.value,
                impact.min,
                rack_impact[2],
                ["End of life is not included in the calculation"],
            )
        else:
            return (
                impact.value,
                rack_impact[1],
                impact.max,
                ["End of life is not included in the calculation"],
            )

    return (
        impact.value * case.units.value,
        impact.min * case.units.min,
        impact.max * case.units.max,
        ["End of life is not included in the calculation"],
    )


def get_impact_constants_blade(impact_type: str) -> Tuple[Impact, Impact]:
    impact_blade_server = Impact(
        value=get_impact_factor(item="case", impact_type=impact_type)["blade"][
            "impact_blade_server"
        ],
        min=get_impact_factor(item="case", impact_type=impact_type)["blade"][
            "impact_blade_server"
        ],
        max=get_impact_factor(item="case", impact_type=impact_type)["blade"][
            "impact_blade_server"
        ],
    )
    impact_blade_16_slots = Impact(
        value=get_impact_factor(item="case", impact_type=impact_type)["blade"][
            "impact_blade_16_slots"
        ],
        min=get_impact_factor(item="case", impact_type=impact_type)["blade"][
            "impact_blade_16_slots"
        ],
        max=get_impact_factor(item="case", impact_type=impact_type)["blade"][
            "impact_blade_16_slots"
        ],
    )

    return impact_blade_server, impact_blade_16_slots


def compute_impact_manufacture_blade(
    impact_blade_server: Impact, impact_blade_16_slots: Impact
) -> ImpactFactor:
    return ImpactFactor(
        value=(impact_blade_16_slots.value / 16) + impact_blade_server.value,
        min=(impact_blade_16_slots.min / 16) + impact_blade_server.min,
        max=(impact_blade_16_slots.max / 16) + impact_blade_server.max,
    )


def iot_functional_blocks_impact_embedded(
    impact_type: str, duration: int, function_blocks: ComponentFunctionalBlock
) -> ComputedImpacts:
    impact = Impact(
        value=get_iot_impact_factor(
            function_blocks.IMPACT_KEY, function_blocks.hsl_level.value, impact_type
        )
        * function_blocks.units.value,
        min=get_iot_impact_factor(
            function_blocks.IMPACT_KEY, function_blocks.hsl_level.value, impact_type
        )
        * function_blocks.units.min,
        max=get_iot_impact_factor(
            function_blocks.IMPACT_KEY, function_blocks.hsl_level.value, impact_type
        )
        * function_blocks.units.max,
    )

    impact.allocate(duration, function_blocks.usage.hours_life_time)

    return impact.value, impact.min, impact.max, []


def hdd_impact_embedded(
    impact_type: str, duration: int, hdd: ComponentHDD
) -> ComputedImpacts:
    impact = Impact(
        value=get_impact_factor(item="hdd", impact_type=impact_type)["impact"]
        * hdd.units.value,
        min=get_impact_factor(item="hdd", impact_type=impact_type)["impact"]
        * hdd.units.min,
        max=get_impact_factor(item="hdd", impact_type=impact_type)["impact"]
        * hdd.units.max,
    )

    impact.allocate(duration, hdd.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def motherboard_impact_embedded(
    impact_type: str, duration: int, motherboard: ComponentMotherboard
) -> ComputedImpacts:
    impact = Impact(
        value=get_impact_factor(item="motherboard", impact_type=impact_type)["impact"]
        * motherboard.units.value,
        min=get_impact_factor(item="motherboard", impact_type=impact_type)["impact"]
        * motherboard.units.min,
        max=get_impact_factor(item="motherboard", impact_type=impact_type)["impact"]
        * motherboard.units.max,
    )

    impact.allocate(duration, motherboard.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def power_supply_impact_embedded(
    impact_type: str, duration: int, power_supply: Component
) -> ComputedImpacts:
    if isinstance(power_supply, ComponentFunctionalBlock):
        return iot_functional_blocks_impact_embedded(
            impact_type, duration, power_supply
        )
    elif isinstance(power_supply, ComponentPowerSupply):
        return server_power_supply_impact_embedded(impact_type, duration, power_supply)
    else:
        raise NotImplementedError


def server_power_supply_impact_embedded(
    impact_type: str, duration: int, power_supply: ComponentPowerSupply
) -> ComputedImpacts:
    impact_factor = Impact(
        value=get_impact_factor(item="power_supply", impact_type=impact_type)["impact"],
        min=get_impact_factor(item="power_supply", impact_type=impact_type)["impact"],
        max=get_impact_factor(item="power_supply", impact_type=impact_type)["impact"],
    )

    impact = Impact(
        value=power_supply.unit_weight.value
        * impact_factor.value
        * power_supply.units.value,
        min=power_supply.unit_weight.min * impact_factor.min * power_supply.units.min,
        max=power_supply.unit_weight.max * impact_factor.max * power_supply.units.max,
    )

    impact.allocate(duration, power_supply.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def ram_impact_embedded(
    impact_type: str, duration: int, ram: ComponentRAM
) -> ComputedImpacts:
    ram_die_impact = Impact(
        value=get_impact_factor(item="ram", impact_type=impact_type)["die_impact"],
        min=get_impact_factor(item="ram", impact_type=impact_type)["die_impact"],
        max=get_impact_factor(item="ram", impact_type=impact_type)["die_impact"],
    )

    ram_impact = Impact(
        value=get_impact_factor(item="ram", impact_type=impact_type)["impact"],
        min=get_impact_factor(item="ram", impact_type=impact_type)["impact"],
        max=get_impact_factor(item="ram", impact_type=impact_type)["impact"],
    )

    impact = Impact(
        value=(
            (ram.capacity.value / ram.density.value) * ram_die_impact.value
            + ram_impact.value
        )
        * ram.units.value,
        min=((ram.capacity.min / ram.density.max) * ram_die_impact.min + ram_impact.min)
        * ram.units.min,
        max=((ram.capacity.max / ram.density.min) * ram_die_impact.max + ram_impact.max)
        * ram.units.max,
    )

    impact.allocate(duration, ram.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def ram_impact_use(
    impact_type: str, duration: int, ram: ComponentRAM
) -> ComputedImpacts:
    impact_factor = ram.usage.elec_factors[impact_type]

    if not ram.usage.avg_power.is_set():
        modeled_consumption = ram.model_power_consumption()
        ram.usage.avg_power.set_completed(
            modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max,
        )

    impact = ImpactFactor(
        value=impact_factor.value
        * (ram.usage.avg_power.value / 1000)
        * ram.usage.use_time_ratio.value
        * duration,
        min=impact_factor.min
        * (ram.usage.avg_power.min / 1000)
        * ram.usage.use_time_ratio.min
        * duration,
        max=impact_factor.max
        * (ram.usage.avg_power.max / 1000)
        * ram.usage.use_time_ratio.max
        * duration,
    )

    return impact.value, impact.min, impact.max, []


def ssd_impact_embedded(
    impact_type: str, duration: int, ssd: ComponentSSD
) -> ComputedImpacts:
    ssd_die_impact = Impact(
        value=get_impact_factor(item="ssd", impact_type=impact_type)["die_impact"],
        min=get_impact_factor(item="ssd", impact_type=impact_type)["die_impact"],
        max=get_impact_factor(item="ssd", impact_type=impact_type)["die_impact"],
    )
    ssd_impact = Impact(
        value=get_impact_factor(item="ssd", impact_type=impact_type)["impact"],
        min=get_impact_factor(item="ssd", impact_type=impact_type)["impact"],
        max=get_impact_factor(item="ssd", impact_type=impact_type)["impact"],
    )

    impact = Impact(
        value=(
            (ssd.capacity.value / ssd.density.value) * ssd_die_impact.value
            + ssd_impact.value
        )
        * ssd.units.value,
        min=((ssd.capacity.min / ssd.density.max) * ssd_die_impact.min + ssd_impact.min)
        * ssd.units.min,
        max=((ssd.capacity.max / ssd.density.min) * ssd_die_impact.max + ssd_impact.max)
        * ssd.units.max,
    )

    impact.allocate(duration, ssd.usage.hours_life_time)

    return (
        impact.value,
        impact.min,
        impact.max,
        ["End of life is not included in the calculation"],
    )


def iot_impact_embedded(
    impact_type: str, duration: int, iot_device: DeviceIoT
) -> ComputedImpacts:
    impacts = []
    min_impacts = []
    max_impacts = []
    warnings = iot_device.WARNINGS

    for component in iot_device.components:
        single_impact = compute_single_impact(
            component, "embedded", impact_type, duration
        )
        impacts.append(single_impact.value)
        min_impacts.append(single_impact.min)
        max_impacts.append(single_impact.max)
        warnings = warnings + single_impact.warnings

    return (
        sum(impacts) * iot_device.units.value,
        sum(min_impacts) * iot_device.units.min,
        sum(max_impacts) * iot_device.units.max,
        warnings,
    )


def iot_impact_use(
    impact_type: str, duration: int, iot_device: DeviceIoT
) -> ComputedImpacts:
    if iot_device.usage.avg_power.value is None:
        raise NotImplementedError

    impact_factor = iot_device.usage.elec_factors[impact_type]
    impact = (
        impact_factor.value
        * (iot_device.usage.avg_power.value / 1000)
        * iot_device.usage.use_time_ratio.value
        * duration
        * iot_device.units.value
    )
    min_impact = (
        impact_factor.min
        * (iot_device.usage.avg_power.min / 1000)
        * iot_device.usage.use_time_ratio.min
        * duration
        * iot_device.units.min
    )
    max_impact = (
        impact_factor.max
        * (iot_device.usage.avg_power.max / 1000)
        * iot_device.usage.use_time_ratio.max
        * duration
        * iot_device.units.max
    )

    return impact, min_impact, max_impact, []


def server_impact_embedded(
    impact_type: str, duration: int, server: DeviceServer
) -> ComputedImpacts:
    impacts = []
    min_impacts = []
    max_impacts = []
    warnings = []

    try:
        for component in server.components:
            component.usage.hours_life_time = server.usage.hours_life_time
            single_impact = compute_single_impact(
                component, "embedded", impact_type, duration
            )

            if single_impact is None:
                raise NotImplementedError

            impacts.append(single_impact.value)
            min_impacts.append(single_impact.min)
            max_impacts.append(single_impact.max)
            warnings = warnings + single_impact.warnings

        return sum(impacts), sum(min_impacts), sum(max_impacts), warnings

    except NotImplementedError:
        impact = Impact(
            value=get_impact_factor(item="SERVER", impact_type=impact_type)["impact"],
            min=get_impact_factor(item="SERVER", impact_type=impact_type)["impact"],
            max=get_impact_factor(item="SERVER", impact_type=impact_type)["impact"],
        )

        warnings = ["Generic data used for impact calculation."]

        impact.allocate(duration, server.usage.hours_life_time)

        return (
            impact.value * server.units.value,
            impact.min * server.units.min,
            impact.max * server.units.max,
            warnings,
        )


def server_impact_use(
    impact_type: str, duration: int, server: DeviceServer
) -> ComputedImpacts:
    impact_factor = server.usage.elec_factors[impact_type]

    if not server.usage.avg_power.is_set():
        modeled_consumption = server.model_power_consumption()
        server.usage.avg_power.set_completed(
            modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max,
        )

    # Compute impacts at component level
    compute_single_impact(server.cpu, USE, impact_type, duration)
    for ram in server.ram:
        compute_single_impact(ram, USE, impact_type, duration)

    impact = (
        impact_factor.value
        * (server.usage.avg_power.value / 1000)
        * server.usage.use_time_ratio.value
        * duration
        * server.units.value
    )
    min_impact = (
        impact_factor.min
        * (server.usage.avg_power.min / 1000)
        * server.usage.use_time_ratio.min
        * duration
        * server.units.min
    )
    max_impact = (
        impact_factor.max
        * (server.usage.avg_power.max / 1000)
        * server.usage.use_time_ratio.max
        * duration
        * server.units.max
    )

    return impact, min_impact, max_impact, []


def cloud_impact_embedded(
    impact_type: str, duration: int, cloud_instance: ServiceCloudInstance
) -> ComputedImpacts:
    impacts = []
    min_impacts = []
    max_impacts = []
    warnings = []
    default_allocation = (
        cloud_instance.vcpu.value / cloud_instance.platform.get_total_vcpu()
    )

    try:
        for component in cloud_instance.platform.components:
            component.usage.hours_life_time = (
                cloud_instance.platform.usage.hours_life_time
            )
            allocation = default_allocation
            if component.NAME == "RAM":
                allocation = (
                    cloud_instance.memory.value
                    / cloud_instance.platform.get_total_memory()
                )
            if component.NAME == "SSD":
                if cloud_instance.ssd_storage.has_value():
                    allocation = (
                        cloud_instance.ssd_storage.value
                        / cloud_instance.platform.get_total_disk_capacity("SSD")
                    )
                else:
                    continue
            if component.NAME == "HDD":
                if cloud_instance.hdd_storage.has_value():
                    allocation = (
                        cloud_instance.hdd_storage.value
                        / cloud_instance.platform.get_total_disk_capacity("HDD")
                    )
                else:
                    continue

            single_impact = compute_single_impact(
                component, "embedded", impact_type, duration, allocation
            )

            if single_impact is None:
                raise NotImplementedError

            impacts.append(single_impact.value)
            min_impacts.append(single_impact.min)
            max_impacts.append(single_impact.max)
            warnings = warnings + single_impact.warnings
        return sum(impacts), sum(min_impacts), sum(max_impacts), warnings

    except NotImplementedError:
        impact = Impact(
            value=get_impact_factor(item="SERVER", impact_type=impact_type)["impact"],
            min=get_impact_factor(item="SERVER", impact_type=impact_type)["impact"],
            max=get_impact_factor(item="SERVER", impact_type=impact_type)["impact"],
        )

        warnings = ["Generic data used for impact calculation."]

        impact.allocate(duration, cloud_instance.platform.usage.hours_life_time)

        return (
            impact.value * default_allocation,
            impact.min * default_allocation,
            impact.max * default_allocation,
            warnings,
        )


def cloud_impact_use(
    impact_type: str, duration: int, cloud_instance: ServiceCloudInstance
) -> ComputedImpacts:
    platform = cloud_instance.platform
    impact_factor = platform.usage.elec_factors[impact_type]

    if not cloud_instance.usage.avg_power.is_set():
        modeled_consumption = cloud_instance.model_power_consumption()
        cloud_instance.usage.avg_power.set_completed(
            value=modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max,
        )

    # Compute impacts at component level
    compute_single_impact(platform.cpu, USE, impact_type, duration)
    for ram in platform.ram:
        compute_single_impact(ram, USE, impact_type, duration)

    impact = (
        impact_factor.value
        * (cloud_instance.usage.avg_power.value / 1000)
        * platform.usage.use_time_ratio.value
        * duration
    )
    min_impact = (
        impact_factor.min
        * (cloud_instance.usage.avg_power.min / 1000)
        * platform.usage.use_time_ratio.min
        * duration
    )
    max_impact = (
        impact_factor.max
        * (cloud_instance.usage.avg_power.max / 1000)
        * platform.usage.use_time_ratio.max
        * duration
    )

    return impact, min_impact, max_impact, []


impacts_functions = {
    "CPU": {"use": cpu_impact_use, "embedded": cpu_impact_embedded},
    "RAM": {"use": ram_impact_use, "embedded": ram_impact_embedded},
    "SSD": {"use": simple_impact_use, "embedded": ssd_impact_embedded},
    "HDD": {"use": simple_impact_use, "embedded": hdd_impact_embedded},
    "POWER_SUPPLY": {
        "use": not_implemented_function,
        "embedded": power_supply_impact_embedded,
    },
    "ASSEMBLY": {"use": not_implemented_function, "embedded": assembly_impact_embedded},
    "CASE": {"use": simple_impact_use, "embedded": casing_impact_embedded},
    "MOTHERBOARD": {"use": simple_impact_use, "embedded": motherboard_impact_embedded},
    "SERVER": {"use": server_impact_use, "embedded": server_impact_embedded},
    "IOT_DEVICE": {"use": iot_impact_use, "embedded": iot_impact_embedded},
    "LAPTOP": {"use": simple_impact_use, "embedded": simple_embedded},
    "DESKTOP": {"use": simple_impact_use, "embedded": simple_embedded},
    "TABLET": {"use": simple_impact_use, "embedded": simple_embedded},
    "SMARTPHONE": {"use": simple_impact_use, "embedded": simple_embedded},
    "TELEVISION": {"use": simple_impact_use, "embedded": simple_embedded},
    "SMARTWATCH": {"use": simple_impact_use, "embedded": ram_impact_embedded},
    "BOX": {"use": simple_impact_use, "embedded": simple_embedded},
    "USB_STICK": {"use": simple_impact_use, "embedded": simple_embedded},
    "EXTERNAL_SSD": {"use": simple_impact_use, "embedded": simple_embedded},
    "EXTERNAL_HDD": {"use": simple_impact_use, "embedded": simple_embedded},
    "MONITOR": {"use": simple_impact_use, "embedded": simple_embedded},
    "CLOUD_INSTANCE": {"use": cloud_impact_use, "embedded": cloud_impact_embedded},
    "ACTUATORS": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "CASING": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "CONNECTIVITY": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "MEMORY": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "OTHERS": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "PCB": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "SECURITY": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "PROCESSING": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "SENSING": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "USER_INTERFACE": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded,
    },
    "VR_CONTROLLER": {"use": simple_impact_use, "embedded": simple_embedded},
    "VR_HEADSET": {"use": simple_impact_use, "embedded": simple_embedded},
}



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.service.archetype import get_component_archetype, get_arch_value


class ComponentHDD(Component):
    NAME = "HDD"

    __DISK_TYPE = "hdd"

    def __init__(
        self, archetype=get_component_archetype(config["default_ssd"], "ssd"), **kwargs
    ):
        super().__init__(archetype=archetype, **kwargs)

        self.capacity = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, "capacity", "default"),
            min=get_arch_value(archetype, "capacity", "default"),
            max=get_arch_value(archetype, "capacity", "default"),
        )



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.service.archetype import get_arch_value, get_component_archetype


class ComponentCase(Component):
    AVAILABLE_CASE_TYPE = ["blade", "rack"]
    NAME = "CASE"

    def __init__(
        self,
        archetype=get_component_archetype(config["default_case"], "case"),
        **kwargs,
    ):
        super().__init__(archetype=archetype, **kwargs)
        self.case_type = Boattribute(
            default=get_arch_value(archetype, "case_type", "default"),
            min=get_arch_value(archetype, "case_type", "min"),
            max=get_arch_value(archetype, "case_type", "max"),
        )



import os

from boaviztapi import config, data_dir
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.service.archetype import (
    get_arch_value,
    get_server_archetype,
    get_cloud_instance_archetype,
)
from boaviztapi.service.factor_provider import (
    get_available_countries,
    get_electrical_impact_factor,
    get_electrical_min_max,
)

_cpu_profile_path = os.path.join(data_dir, "consumption_profile/cpu/cpu_profile.csv")
_cloud_profile_path = os.path.join(
    data_dir, "consumption_profile/cloud/cpu_profile.csv"
)
_server_profile_path = os.path.join(
    data_dir, "consumption_profile/server/server_profile.csv"
)


class ModelUsage:
    _DAYS_IN_HOURS = 24
    _YEARS_IN_HOURS = 24 * 365

    def __init__(self, archetype, **kwargs):
        self.archetype = archetype
        self.avg_power = Boattribute(
            unit="W",
            default=get_arch_value(archetype, "avg_power", "default"),
            min=get_arch_value(archetype, "avg_power", "min"),
            max=get_arch_value(archetype, "avg_power", "max"),
        )
        self.time_workload = Boattribute(
            unit="%",
            default=get_arch_value(archetype, "time_workload", "default"),
            min=get_arch_value(archetype, "time_workload", "min"),
            max=get_arch_value(archetype, "time_workload", "max"),
        )
        self.consumption_profile = None
        self.usage_location = Boattribute(
            unit="CodSP3 - NCS Country Codes - NATO",
            default=get_arch_value(archetype, "usage_location", "default"),
            min=get_arch_value(archetype, "usage_location", "min"),
            max=get_arch_value(archetype, "usage_location", "max"),
        )
        self.use_time_ratio = Boattribute(
            unit="/1",
            default=get_arch_value(archetype, "use_time_ratio", "default"),
            min=get_arch_value(archetype, "use_time_ratio", "min"),
            max=get_arch_value(archetype, "use_time_ratio", "max"),
        )
        self.hours_life_time = Boattribute(
            unit="hours",
            default=get_arch_value(archetype, "hours_life_time", "default"),
            min=get_arch_value(archetype, "hours_life_time", "min"),
            max=get_arch_value(archetype, "hours_life_time", "max"),
        )
        self.elec_factors = {
            "gwp": Boattribute(
                unit="kg CO2eq/kWh", complete_function=self._complete_gwp
            ),
            "adp": Boattribute(
                unit="kg Sbeq/kWh", complete_function=self._complete_adp
            ),
            "pe": Boattribute(unit="MJ/kWh", complete_function=self._complete_pe),
            "gwppb": Boattribute(
                unit="kg CO2eq/kWh", complete_function=self._complete_gwppb
            ),
            "gwppf": Boattribute(
                unit="kg CO2eq/kWh", complete_function=self._complete_gwppf
            ),
            "gwpplu": Boattribute(
                unit="kg CO2eq/kWh", complete_function=self._complete_gwpplu
            ),
            "ir": Boattribute(
                unit="kg U235eq/kWh", complete_function=self._complete_ir
            ),
            "lu": Boattribute(
                unit="No dimension/kWh", complete_function=self._complete_lu
            ),
            "odp": Boattribute(
                unit="kg CFC-11eq/kWh", complete_function=self._complete_odp
            ),
            "pm": Boattribute(
                unit="Disease occurrence/kWh", complete_function=self._complete_pm
            ),
            "pocp": Boattribute(
                unit="kg NMVOCeq/kWh", complete_function=self._complete_pocp
            ),
            "wu": Boattribute(unit="m3eq/kWh", complete_function=self._complete_wu),
            "mips": Boattribute(unit="kg/kWh", complete_function=self._complete_mips),
            "adpe": Boattribute(
                unit="kg Sbeq/kWh", complete_function=self._complete_adpe
            ),
            "adpf": Boattribute(unit="MJ/kWh", complete_function=self._complete_adpf),
            "ap": Boattribute(unit="mol H+eq/kWh", complete_function=self._complete_ap),
            "ctue": Boattribute(unit="CTUe/kWh", complete_function=self._complete_ctue),
            "ctuh_c": Boattribute(
                unit="CTUh/kWh", complete_function=self._complete_ctuh_c
            ),
            "ctuh_nc": Boattribute(
                unit="CTUh/kWh", complete_function=self._complete_ctuh_nc
            ),
            "epf": Boattribute(unit="kg Peq/kWh", complete_function=self._complete_epf),
            "epm": Boattribute(unit="kg Neq/kWh", complete_function=self._complete_epm),
            "ept": Boattribute(
                unit="mol Neq/kWh", complete_function=self._complete_ept
            ),
        }

    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value

    def _complete_impact_factor(self, impact_criteria, impact_criteria_proxy):
        if impact_criteria is None:
            raise NotImplementedError

        if not self.usage_location.has_value():
            self.usage_location.set_default(config["default_location"])

        if self.usage_location.value not in get_available_countries(reverse=True):
            raise NotImplementedError

        factor = get_electrical_impact_factor(
            self.usage_location.value, impact_criteria_proxy
        )

        if self.usage_location.is_default():
            self.elec_factors.get(impact_criteria).set_default(
                factor["value"], source=str(factor["source"])
            )
            self.elec_factors.get(impact_criteria).min = float(
                get_electrical_min_max(impact_criteria_proxy, "min")
            )
            self.elec_factors.get(impact_criteria).max = float(
                get_electrical_min_max(impact_criteria_proxy, "max")
            )
        else:
            self.elec_factors.get(impact_criteria).set_completed(
                factor["value"],
                source=str(factor["source"]),
                min=factor["value"],
                max=factor["value"],
            )

    def _complete_gwp(self):
        self._complete_impact_factor("gwp", "gwp")

    def _complete_adp(self):
        self._complete_impact_factor("adp", "adpe")

    def _complete_pe(self):
        self._complete_impact_factor("pe", "pe")

    def _complete_gwppb(self):
        self._complete_impact_factor("gwppb", "gwppb")

    def _complete_gwppf(self):
        self._complete_impact_factor("gwppf", "gwppf")

    def _complete_gwpplu(self):
        self._complete_impact_factor("gwpplu", "gwpplu")

    def _complete_ir(self):
        self._complete_impact_factor("ir", "ir")

    def _complete_lu(self):
        self._complete_impact_factor("lu", "lu")

    def _complete_odp(self):
        self._complete_impact_factor("odp", "odp")

    def _complete_pm(self):
        self._complete_impact_factor("pm", "pm")

    def _complete_pocp(self):
        self._complete_impact_factor("pocp", "pocp")

    def _complete_wu(self):
        self._complete_impact_factor("wu", "wu")

    def _complete_mips(self):
        self._complete_impact_factor("mips", "mips")

    def _complete_adpe(self):
        self._complete_impact_factor("adpe", "adpe")

    def _complete_adpf(self):
        self._complete_impact_factor("adpf", "adpf")

    def _complete_ap(self):
        self._complete_impact_factor("ap", "ap")

    def _complete_ctue(self):
        self._complete_impact_factor("ctue", "ctue")

    def _complete_ctuh_c(self):
        self._complete_impact_factor("ctuh_c", "ctuh_c")

    def _complete_ctuh_nc(self):
        self._complete_impact_factor("ctuh_nc", "ctuh_nc")

    def _complete_epf(self):
        self._complete_impact_factor("epf", "epf")

    def _complete_epm(self):
        self._complete_impact_factor("epm", "epm")

    def _complete_ept(self):
        self._complete_impact_factor("ept", "ept")


class ModelUsageServer(ModelUsage):
    def __init__(
        self,
        archetype=get_server_archetype(config["default_server"]).get("USAGE"),
        **kwargs,
    ):
        super().__init__(archetype=archetype, **kwargs)

        self.other_consumption_ratio = Boattribute(
            unit="ratio /1",
            default=get_arch_value(archetype, "other_consumption_ratio", "default"),
            min=get_arch_value(archetype, "other_consumption_ratio", "min"),
            max=get_arch_value(archetype, "other_consumption_ratio", "max"),
        )


class ModelUsageCloud(ModelUsageServer):
    def __init__(
        self,
        archetype=get_cloud_instance_archetype(
            config["default_cloud_instance"], config["default_cloud_provider"]
        ).get("USAGE"),
        **kwargs,
    ):
        super().__init__(archetype=archetype, **kwargs)
        self.instance_per_server = Boattribute(
            default=get_arch_value(archetype, "instance_per_server", "default"),
            min=get_arch_value(archetype, "instance_per_server", "min"),
            max=get_arch_value(archetype, "instance_per_server", "max"),
        )



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.device.server import DeviceServer
from boaviztapi.model.impact import Assessable, ImpactFactor
from boaviztapi.model.usage import ModelUsage
from boaviztapi.service.archetype import (
    get_server_archetype,
    get_cloud_instance_archetype,
    get_arch_value,
)


class Service(Assessable):
    def __init__(self, archetype=None, **kwargs):
        super().__init__(**kwargs)
        self.units = Boattribute(default=1, min=1, max=1)
        self.archetype = archetype
        self._usage = None
        self._impacts = {}

    @property
    def usage(self) -> ModelUsage:
        return self._usage

    @usage.setter
    def usage(self, value: int) -> None:
        self._usage = value


class ServiceCloudInstance(Service):
    NAME = "CLOUD_INSTANCE"

    def __init__(
        self,
        archetype=get_cloud_instance_archetype(
            config["default_cloud_instance"], config["default_cloud_provider"]
        ),
        **kwargs,
    ):
        super().__init__(archetype=archetype, **kwargs)

        self._platform = DeviceServer(
            archetype=get_server_archetype(
                archetype_name=get_arch_value(archetype, "platform", "default")
            )
        )
        self.vcpu = Boattribute(
            default=get_arch_value(archetype, "vcpu", "default"),
            min=get_arch_value(archetype, "vcpu", "min"),
            max=get_arch_value(archetype, "vcpu", "max"),
        )
        self.memory = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, "memory", "default"),
            min=get_arch_value(archetype, "memory", "min"),
            max=get_arch_value(archetype, "memory", "max"),
        )
        self.hdd_storage = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, "hdd_storage", "default"),
            min=get_arch_value(archetype, "hdd_storage", "min"),
            max=get_arch_value(archetype, "hdd_storage", "max"),
        )
        self.ssd_storage = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, "ssd_storage", "default"),
            min=get_arch_value(archetype, "ssd_storage", "min"),
            max=get_arch_value(archetype, "ssd_storage", "max"),
        )

        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)

    @property
    def platform(self) -> DeviceServer:
        if self._platform is None:
            self._platform = DeviceServer(
                archetype=get_server_archetype(
                    archetype_name=self.archetype["platform"]
                )
            )
        return self._platform

    @platform.setter
    def platform(self, value: DeviceServer) -> None:
        self._platform = value

    def model_power_consumption(self):
        vcpu_allocation = self.vcpu.value / self.platform.get_total_vcpu()
        ram_allocation = self.memory.value / self.platform.get_total_memory()

        total_cpu_consumption = self.platform.cpu.model_power_consumption()
        self.platform.cpu.usage.avg_power.set_completed(
            value=total_cpu_consumption.value * vcpu_allocation,
            min=total_cpu_consumption.min * vcpu_allocation,
            max=total_cpu_consumption.max * vcpu_allocation,
        )

        total_conso_ram = ImpactFactor(value=0, min=0, max=0)

        for ram in self.platform.ram:
            ram_consumption = ram.model_power_consumption()

            ram.usage.avg_power.set_completed(
                value=ram_consumption.value * ram_allocation,
                min=ram_consumption.min * ram_allocation,
                max=ram_consumption.max * ram_allocation,
            )

            total_conso_ram.value = total_conso_ram.value + ram.usage.avg_power.value
            total_conso_ram.min = total_conso_ram.min + ram.usage.avg_power.min
            total_conso_ram.max = total_conso_ram.max + ram.usage.avg_power.max

        return ImpactFactor(
            value=(self.platform.cpu.usage.avg_power.value + total_conso_ram.value)
            * (1 + self.platform.usage.other_consumption_ratio.value),
            min=(self.platform.cpu.usage.avg_power.min + total_conso_ram.min)
            * (1 + self.platform.usage.other_consumption_ratio.min),
            max=(self.platform.cpu.usage.avg_power.max + total_conso_ram.max)
            * (1 + self.platform.usage.other_consumption_ratio.max),
        )

    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value



import pandas as pd
from pandas.core.series import Series
from rapidfuzz import process, fuzz
from typing import Tuple, Union

from boaviztapi import config


def fuzzymatch_attr_from_cpu_name(
    cpu_name: str, df: pd.DataFrame
) -> Union[Tuple[str, str, str, str, int, int, int, int, str, str], None]:
    cpu_name = cpu_name.lower()
    score = df["name"].str.lower().apply(lambda x: fuzz.token_set_ratio(x, cpu_name))
    max_score = score.max()
    if max_score <= config["cpu_name_fuzzymatch_threshold"]:
        return None
    else:
        best = df.iloc[score.idxmax()]
        best = best.mask(best.isnull(), None)  # replace all NaN by None

        def safe_int(x):
            return x if x is None else int(x)  # float to int but keep None

        return (
            best["name"],  # .name is reserved by pandas for indexes
            best.manufacturer,
            best.code_name,
            best.model_range,
            safe_int(best.tdp),
            safe_int(best.cores),
            safe_int(best.threads),
            safe_int(best.total_die_size),
            best.total_die_size_source,
            best.source,
        )


def fuzzymatch_attr_from_pdf(name: str, attr: str, pdf: pd.DataFrame) -> str:
    name_list = list(pdf[attr].unique())

    result = process.extractOne(name, name_list, scorer=fuzz.WRatio)
    if result is not None:
        result = result[0] if result[1] > 79.0 else None
    return result or None


def fuzzymatch(s: pd.Series, value: str, threshold: float = 90.0) -> pd.Series:
    return s.apply(
        lambda x: fuzz.token_sort_ratio(x.lower(), value.lower()) >= threshold
    )


def pandas() -> None:
    Series.fuzzymatch = fuzzymatch



server_impact_by_model_description = (
    "# ✔ ️Server impacts from model name\n"
    "Retrieve the impacts of a given server archetype.\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "🔌 Usage\n\n"
    "📋 Archetype: "
    "Uses the classic server impacts router with a pre-registered archetype \n\n"
    "⏬ Allocation"
)

server_impact_by_config_description = (
    "# ✔️ Server impacts from configuration\n"
    "Retrieve the impacts of a given server configuration.\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "* 📈 Modeled\n\n"
    "📋 Archetype\n\n"
    "⏬ Allocation"
)

all_archetype_servers = "# ✔️ Get all the available server archetype\n"
all_archetype_components = (
    "# ✔️ Get all the available component archetype for a given component name\n"
)
all_archetype_user_terminals = (
    "# ✔️ Get all the available user terminal archetype for a given user terminal name\n"
)
all_terminal_categories = "# ✔️ Get all the available user terminal router\n"
all_peripheral_categories = "# ✔️ Get all the available user peripheral router\n"
all_user_terminal_subcategories = (
    "# ✔️ Get all the available user terminal subcategories\n"
)
all_default_usage_values = "# ✔️ Get all default usage values for a given user terminal category and subcategory\n"

get_archetype_config_desc = "# ✔️ Get the configuration of a given archetype\n"
get_instance_config = "# ✔️ Get the configuration of a given instance\n"

cpu_description = (
    "# ✔ ️CPU impacts from configuration\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "<h3>cpu<sub>manuf<sub><em>criteria</em></sub></sub> = ( "
    "cpu<sub>core<sub>units</sub></sub> x cpu<sub>diesize</sub> + 0,"
    "491 ) x cpu<sub>manuf_die<sub><em>criteria</em></sub></sub> + "
    "cpu<sub>manuf_base<sub><em>criteria</em></sub></sub></h3> "
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "* 📈 Modeled\n\n"
    "⏬ Allocation"
)

ssd_description = (
    "# ✔ ️SSD impacts from configuration\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "<h3>ssd<sub>manuf<sub><em>criteria</em></sub></sub> =  ( ssd<sub>size</sub> "
    "ssd<sub>density</sub> ) x ssd<sub>manuf_die<sub><em>criteria</em></sub></sub> + "
    "ssd<sub>manuf_base<sub><em>criteria</em></sub></sub></h3>"
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "⏬ Allocation"
)

hdd_description = (
    "# ✔ ️HDD impacts from configuration\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "The impacts values are set by default"
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "⏬ Allocation"
)

ram_description = (
    "# ✔️ RAM impacts from configuration\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "<h3>ram<sub>manuf<sub><em>criteria</em></sub></sub> =( ram<sub>size</sub> "
    "/ ram<sub>density</sub> ) x ram<sub>manuf_die<sub><em>criteria</em></sub></sub> + "
    "ram<sub>manuf_base<sub><em>criteria</em></sub></sub> </h3> "
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "* 📈 Modeled\n\n"
    "⏬ Allocation"
)

motherboard_description = (
    "# ✔ ️Motherboard impacts from configuration\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "The impacts values are set by default"
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "⏬ Allocation"
)

power_supply_description = (
    "# ✔ ️Power supply impacts from configuration\n" + "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "<h3>psu<sub>manuf<sub><em>criteria</em></sub></sub> = psu<sub>unit<sub>weight</sub></sub>"
    " x psu<sub>manuf_weight<sub><em>criteria</em></sub></sub></h3> "
    "🔌 Usage\n\n"
    "* ⏺️  Given : shouldn't be used\n\n"
    "⏬ Allocation"
)

case_description = (
    "# ✔ ️Case impacts from configuration\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "🔌 Usage\n\n"
    "* ⏺️  Given : when the enclosure consumes energy \n\n"
    "⏬ Allocation"
)

cloud_provider_description = (
    "# ✔ ️Cloud instance impacts from provider, instance type and usage \n"
    "Retrieve the impacts of a given Cloud instance and usage.\n\n"
    "### Features\n\n"
    "📋 Provider \n\n"
    "Name of the cloud provider. You can retrieve the [list here]("
    "#/cloud_instance/server_get_all_cloud_providers).\n\n"
    "📋 Instance type \n\n"
    "Name of the chosen instance. You can retrieve the [list here]("
    "#/cloud/server_get_archetype_name_v1_cloud_all_aws_instances_get).\n\n"
    "👄 Verbose\n\n"
    "🔨 Embedded\n\n"
    "🔌 Usage \n\n"
    "* 📈 Modeled\n\n"
    "📋 Archetype : The configuration is set by the API, only usage is given by the user\n\n"
    "⏬ Allocation"
)

all_default_cloud_instances = (
    "# ✔ ️Get all the available instances for a given Cloud provider\n"
    "📜 Return the name of all pre-registered instances for the Cloud provider"
)

all_default_cloud_providers = (
    "# ✔ ️Get all the available Cloud providers\n"
    "📜 Return the names of all pre-registered Cloud providers"
)

country_code = "# ✔ ️Get all the available countries with their trigram code (*usage:{usage_location: 'FRA'}*)\n"
cpu_family = (
    "# ✔ ️Get all the available cpu family in the API (*cpu:{family:'skylake'}*)\n"
)
cpu_model_range = "# ✔ ️Get all the available cpu family in the API (*cpu:{model_range:'xeon platinum'}*)\n"
ssd_manufacturer = "# ✔ ️Get all the available ssd manufacturer in the API (*ssd:{manufacturer:'samsung'}*)\n"
ram_manufacturer = "# ✔ ️Get all the available ram manufacturer in the API (*ram:{manufacturer:'samsung'}*)\n"
case_type = "# ✔ ️Get all the available case type in the API (*model:{case:'blade'}*)\n"
name_to_cpu = "# ✔ ️Complete a cpu attributes from a cpu name\n"
cpu_names = "# ✔ ️Get all the available cpu name in the API (*cpu:{name:'intel xeon platinum 8175m'}*)\n"
impacts_criteria = "# ✔ ️Get all the available criteria for the impacts calculation\n"


terminal_description = (
    "# ✔ Terminal impacts\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "The impacts values are fix"
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "* 📋 Archetype\n\n"
    "⏬ Allocation"
)


peripheral_description = (
    "# ✔ Peripheral impacts\n"
    "### Features\n\n"
    "👄 Verbose\n\n"
    "🔃 Auto-complete\n\n"
    "🔨 Embedded\n\n"
    "The impacts values are fix"
    "🔌 Usage\n\n"
    "* ⏺️  Given\n\n"
    "* 📋 Archetype\n\n"
    "⏬ Allocation"
)



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.service.archetype import get_component_archetype, get_arch_value


class ComponentPowerSupply(Component):
    NAME = "POWER_SUPPLY"

    def __init__(
        self,
        archetype=get_component_archetype(
            config["default_power_supply"], "power_supply"
        ),
        **kwargs,
    ):
        super().__init__(archetype=archetype, **kwargs)

        self.unit_weight = Boattribute(
            unit="kg",
            default=get_arch_value(archetype, "unit_weight", "default"),
            min=get_arch_value(archetype, "unit_weight", "min"),
            max=get_arch_value(archetype, "unit_weight", "max"),
        )



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.device import Device
from boaviztapi.model.component import Component
from boaviztapi.model.services.cloud_instance import ServiceCloudInstance, Service


def verbose_cloud(
    cloud_instance: ServiceCloudInstance,
    selected_criteria=config["default_criteria"],
    duration=config["default_duration"],
):
    json_output = {
        **iter_boattribute(cloud_instance),
        **verbose_usage(cloud_instance),
        **verbose_device(
            cloud_instance.platform,
            selected_criteria=selected_criteria,
            duration=duration,
        ),
    }
    return json_output


def verbose_device(
    device: Device,
    selected_criteria=config["default_criteria"],
    duration=config["default_duration"],
):
    json_output = {"duration": {"value": duration, "unit": "hours"}}
    for component in device.components:
        component.usage.hours_life_time.set_completed(
            device.usage.hours_life_time.value,
            min=device.usage.hours_life_time.min,
            max=device.usage.hours_life_time.max,
            source="from device",
        )
        if f"{component.NAME}-1" in json_output:
            i = 2
            while f"{component.NAME}-{i}" in json_output:
                i += 1
            key = f"{component.NAME}-{i}"
        else:
            key = f"{component.NAME}-1"

        json_output[key] = verbose_component(component, selected_criteria, duration)

    json_output = {**json_output, **verbose_usage(device), **iter_boattribute(device)}

    return json_output


def verbose_usage(device: [Device, Component, Service]):
    json_output = {**iter_boattribute(device.usage)}
    if device.usage.consumption_profile is not None:
        if device.usage.consumption_profile.workloads.is_set():
            json_output["workloads"] = (
                device.usage.consumption_profile.workloads.to_json()
            )
            json_output["workloads"]["value"] = [
                {
                    "load_percentage": workload.load_percentage,
                    "power_watt": workload.power_watt,
                }
                for workload in json_output["workloads"]["value"]
            ]
        if device.usage.consumption_profile.params.is_set():
            json_output["params"] = device.usage.consumption_profile.params.to_json()
    for elec in device.usage.elec_factors:
        if device.usage.elec_factors[elec].is_set():
            json_output[f"{elec}_factor"] = device.usage.elec_factors[elec].to_json()

    return json_output


def verbose_component(
    component: Component,
    selected_criteria=config["default_criteria"],
    duration=config["default_duration"],
):
    json_output = {
        "impacts": component.get_impacts(selected_criteria),
        **iter_boattribute(component),
        "duration": {"value": duration, "unit": "hours"},
    }

    if component.usage.avg_power.is_set():
        json_output = {**json_output, **verbose_usage(component)}

    return json_output


def iter_boattribute(element):
    json_output = {}
    for attr, val in element.__iter__():
        if not isinstance(val, Boattribute):
            continue
        if val.is_set():
            json_output[attr] = val.to_json()
    return json_output



import os

import pandas as pd

import boaviztapi.utils.roundit as rd
from boaviztapi import config, data_dir
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.model.consumption_profile.consumption_profile import (
    RAMConsumptionProfileModel,
)
from boaviztapi.model.impact import ImpactFactor
from boaviztapi.service.archetype import get_arch_value, get_component_archetype
from boaviztapi.utils.fuzzymatch import fuzzymatch_attr_from_pdf


class ComponentRAM(Component):
    NAME = "RAM"

    _ram_df = pd.read_csv(os.path.join(data_dir, "crowdsourcing/ram_manufacture.csv"))

    def __init__(
        self, archetype=get_component_archetype(config["default_ram"], "ram"), **kwargs
    ):
        super().__init__(archetype=archetype, **kwargs)

        self.process = Boattribute(
            default=get_arch_value(archetype, "process", "default"),
            min=get_arch_value(archetype, "process", "min"),
            max=get_arch_value(archetype, "process", "max"),
        )
        self.manufacturer = Boattribute(
            default=get_arch_value(archetype, "manufacturer", "default"),
            min=get_arch_value(archetype, "manufacturer", "min"),
            max=get_arch_value(archetype, "manufacturer", "max"),
        )
        self.capacity = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, "capacity", "default"),
            min=get_arch_value(archetype, "capacity", "min"),
            max=get_arch_value(archetype, "capacity", "max"),
        )
        self.density = Boattribute(
            complete_function=self._complete_density,
            unit="GB/cm2",
            default=get_arch_value(archetype, "density", "default"),
            min=get_arch_value(archetype, "density", "min"),
            max=get_arch_value(archetype, "density", "max"),
        )

    # IMPACT COMPUTATION

    def model_power_consumption(
        self,
    ) -> ImpactFactor:
        self.usage.consumption_profile = RAMConsumptionProfileModel()
        self.usage.consumption_profile.compute_consumption_profile_model(
            ram_capacity=self.capacity.value
        )

        if type(self.usage.time_workload.value) in (float, int):
            self.usage.avg_power.set_completed(
                self.usage.consumption_profile.apply_consumption_profile(
                    self.usage.time_workload.value
                )
            )
        else:
            self.usage.avg_power.set_completed(
                self.usage.consumption_profile.apply_multiple_workloads(
                    self.usage.time_workload.value
                )
            )

        return ImpactFactor(
            value=rd.round_to_sigfig(self.usage.avg_power.value, 5) * self.units.value,
            min=rd.round_to_sigfig(self.usage.avg_power.value, 5) * self.units.min,
            max=rd.round_to_sigfig(self.usage.avg_power.value, 5) * self.units.max,
        )

    # COMPLETION
    def _complete_density(self):
        sub = self._ram_df

        if self.manufacturer.has_value():
            corrected_manufacturer = fuzzymatch_attr_from_pdf(
                self.manufacturer.value, "manufacturer", sub
            )
            if corrected_manufacturer is not None:
                sub = sub[sub["manufacturer"] == corrected_manufacturer]
                if corrected_manufacturer != self.manufacturer.value:
                    self.manufacturer.set_changed(corrected_manufacturer)

        if self.process.has_value():
            sub = sub[sub["process"] == self.process.value]

        if len(sub) == 1:
            self.density.set_completed(
                float(sub["density"]),
                source=str(sub["manufacturer"].iloc[0]),
                min=float(sub["density"]),
                max=float(sub["density"]),
            )

        elif (
            len(sub) == 0 or len(sub) == len(self._ram_df)
        ) and self.density.has_value():
            return

        else:
            self.density.set_completed(
                float(sub["density"].mean()),
                source="Average of " + str(len(sub)) + " rows",
                min=float(sub["density"].min()),
                max=float(sub["density"].max()),
            )



import dataclasses
import math
import os
from typing import Dict, Optional, List, Tuple, Union

import numpy as np
import pandas as pd
from scipy.optimize import curve_fit

import boaviztapi.utils.fuzzymatch as fuzzymatch
from boaviztapi import config, data_dir
from boaviztapi.dto.usage.usage import WorkloadTime
from boaviztapi.model.boattribute import Boattribute, Status
from boaviztapi.service.archetype import get_component_archetype, get_arch_value

fuzzymatch.pandas()

_cpu_profile_consumption_df = pd.read_csv(
    os.path.join(data_dir, "consumption_profile/cpu/cpu_profile.csv")
)

MIN_POWER = 1  # Minimal power is 1 W


class ConsumptionProfileModel:
    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value


class RAMConsumptionProfileModel(ConsumptionProfileModel):
    ram_electrical_factor_per_go = 0.284

    def __init__(
        self,
        archetype=get_component_archetype(config["default_ram"], "ram").get(
            "consumption_profile"
        ),
    ):
        self.workloads = Boattribute(unit="workload_rate:W")
        self.params = Boattribute()

    def compute_consumption_profile_model(self, ram_capacity) -> int:
        self.params.value = {"a": self.ram_electrical_factor_per_go * ram_capacity}
        self.params.status = Status.COMPLETED
        self.params.source = (
            f"(ram_electrical_factor_per_go : {self.ram_electrical_factor_per_go}) * ("
            f"ram_capacity: {ram_capacity}) "
        )
        return self.params.value

    def apply_consumption_profile(self, load_percentage: float) -> float:
        return self.params.value["a"]

    def apply_multiple_workloads(self, time_workload: List[WorkloadTime]) -> float:
        total = 0
        for workload in time_workload:
            total += (workload.time_percentage / 100) * self.apply_consumption_profile(
                workload.load_percentage
            )
        return total


class CPUConsumptionProfileModel(ConsumptionProfileModel):
    _TDP_RATIOS_WORKLOAD = [0, 10, 50, 100]
    _TDP_RATIOS = [0.12, 0.32, 0.75, 1.02]

    _DEFAULT_MODEL_BOUNDS = (
        [0, 0, 0, -math.inf],
        [math.inf, math.inf, math.inf, math.inf],
    )
    _MODEL_PARAM_NAME = ["a", "b", "c", "d"]

    def __init__(
        self,
        archetype=get_component_archetype(config["default_cpu"], "cpu").get(
            "CONSUMPTION_PROFILE"
        ),
    ):
        self.workloads = Boattribute(unit="workload_rate:W")
        self.params = Boattribute(
            default=get_arch_value(archetype, "params", "default"),
            min=get_arch_value(archetype, "params", "min"),
            max=get_arch_value(archetype, "params", "max"),
        )

    @property
    def list_workloads(self) -> Tuple[List[float], List[float]]:
        load = [item.load_percentage for item in self.workloads.value]
        power = [item.power_watt for item in self.workloads.value]
        return load, power

    def apply_consumption_profile(self, load_percentage: float) -> float:
        power = self.__log_model(
            load_percentage,
            self.params.value["a"],
            self.params.value["b"],
            self.params.value["c"],
            self.params.value["d"],
        )
        if power < MIN_POWER:
            self.params.add_warning(
                "Fitted CPU consumption profile model yielded very low or negative power values, "
                "this can be caused by wrong input data or model initialization. Power consumption "
                "and usage impacts of the CPU might be false."
            )
        return max(power, MIN_POWER)

    def apply_multiple_workloads(self, time_workload: List[WorkloadTime]) -> float:
        total = 0
        for workload in time_workload:
            total += (workload.time_percentage / 100) * self.apply_consumption_profile(
                workload.load_percentage
            )
        return total

    def compute_consumption_profile_model(
        self,
        cpu_manufacturer: str = None,
        cpu_model_range: str = None,
        cpu_tdp: int = None,
    ) -> Union[Dict[str, float], None]:
        model = self.lookup_consumption_profile(cpu_manufacturer, cpu_model_range)
        if self.workloads.is_set():
            model = self.__compute_model_adaptation(
                base_model=model or self.params.default
            )
            self.params.set_completed(model, source="From workload")

        elif cpu_tdp is not None:
            model = self.__compute_model_adaptation_with_tdp(
                base_model=model or self.params.default, cpu_tdp=cpu_tdp
            )
            self.params.set_completed(model, source="From TDP")

        elif cpu_model_range is not None:
            self.params.set_completed(model, source="From CPU model range")

        return self.params.value

    def __compute_model_adaptation_with_tdp(
        self, base_model: Dict[str, float], cpu_tdp: float
    ) -> Dict[str, float]:
        @dataclasses.dataclass
        class _TDPWorkloadPower:
            load_percentage: float = None
            power_watt: float = None

        self.workloads.set_completed(
            [
                _TDPWorkloadPower(load_percentage=w, power_watt=cpu_tdp * r)
                for w, r in zip(self._TDP_RATIOS_WORKLOAD, self._TDP_RATIOS)
            ]
        )
        return self.__compute_model_adaptation(base_model=base_model)

    def __compute_model_adaptation(
        self, base_model: Dict[str, float]
    ) -> Dict[str, float]:
        base_model_list = self.__model_dict_to_list(base_model)
        bounds = self.__adapt_model_bounds(base_model_list)
        x_data, y_data = self.list_workloads
        popt, _ = curve_fit(
            f=self.__log_model,
            xdata=x_data,
            ydata=y_data,
            p0=base_model_list,
            bounds=bounds,
        )
        return self.__model_list_to_dict(popt.tolist())

    def __adapt_model_bounds(
        self, base_model_list: List[float]
    ) -> Tuple[List[float], List[float]]:
        default_lower_bounds, default_upper_bounds = self._DEFAULT_MODEL_BOUNDS
        lower_bounds, upper_bounds = [], []
        for lower_b, upper_b, model_param in zip(
            default_lower_bounds, default_upper_bounds, base_model_list
        ):
            lower_bounds.append(max(lower_b, model_param - abs(2 * model_param)))
            upper_bounds.append(min(upper_b, model_param + abs(2 * model_param)))
        return lower_bounds, upper_bounds

    def __model_dict_to_list(self, model: Dict[str, float]) -> List[float]:
        return [model[param_name] for param_name in self._MODEL_PARAM_NAME]

    def __model_list_to_dict(self, model: List[float]) -> Dict[str, float]:
        return {
            param_name: param
            for param_name, param in zip(self._MODEL_PARAM_NAME, model)
        }

    @staticmethod
    def __log_model(x: float, a: float, b: float, c: float, d: float) -> float:
        return a * np.log(b * (x + c)) + d

    @staticmethod
    def lookup_consumption_profile(
        cpu_manufacturer: str = None, cpu_model_range: str = None
    ) -> Optional[Dict[str, float]]:
        sub = _cpu_profile_consumption_df

        if cpu_manufacturer is not None:
            tmp = sub[sub["manufacturer"].fuzzymatch(cpu_manufacturer)]
            if len(tmp) > 0:
                sub = tmp.copy()

        if cpu_model_range is not None:
            tmp = sub[sub["model_range"].fuzzymatch(cpu_model_range)]
            if len(tmp) > 0:
                sub = tmp.copy()

        if len(sub) == 1:
            row = sub.iloc[0]
            return {
                "a": row.a,
                "b": row.b,
                "c": row.c,
                "d": row.d,
            }
